
#ifndef TEST_H
#define TEST_H

#define LAZY_PTR
#ifdef LAZY_PTR
//#define NO_CONST
#endif

#include <memory>

#include <odb/core.hxx>
#include <odb/lazy-ptr.hxx>

class Employee;
typedef std::shared_ptr<Employee> EmployeePtr;
typedef std::shared_ptr<const Employee> ConstEmployeePtr;

class Employer;
typedef std::shared_ptr<Employer> EmployerPtr;
typedef std::shared_ptr<const Employer> ConstEmployerPtr;

#pragma db object transient session
class Employee
    : public std::enable_shared_from_this<Employee>
{
public:
#ifdef NO_CONST
    static EmployeePtr create(EmployerPtr const& employer, std::string const& name);
#else
    static EmployeePtr create(ConstEmployerPtr const& employer, std::string const& name);
#endif

    unsigned int id() const;

#ifdef NO_CONST
    EmployerPtr employer() const;
#else
    ConstEmployerPtr employer() const;
#endif

    std::string& name() const;
    void name(std::string const&);

private:
    #pragma db id auto
    #pragma db member(id) virtual(unsigned int) get(this.impl().m_id) set(const_cast< unsigned int & >(this.impl().m_id))
#ifdef LAZY_PTR
#ifdef NO_CONST
    typedef odb::lazy_weak_ptr< Employer > employer_type;
#else
    typedef odb::lazy_weak_ptr< const Employer > employer_type;
#endif
#else
    typedef std::weak_ptr< const Employer > employer_type;
#endif
    #pragma db inverse(employees)
    #pragma db member(employer) virtual(employer_type) access(this.impl().m_employer)
    #pragma db member(name) virtual(std::string) access(this.impl().m_name)

    friend class odb::access;

public:
    ~Employee();

protected:
    class Impl;
    Employee();
    Employee(Impl *);

    const Impl & impl() const;
    Impl & impl();

private:
    Impl * const impl_;
};

typedef std::vector< EmployeePtr > Employees;
#pragma db value(Employees) value_not_null

#pragma db object transient session
class Employer
    : public std::enable_shared_from_this<Employer>
{
public:
    static EmployerPtr create(std::string const& name);

    unsigned int id() const;

    std::string& name() const;
    void name(std::string const&);

    Employees const & employees() const;
    void add_employee(EmployeePtr const& );

private:
    #pragma db id auto
    #pragma db member(id) virtual(unsigned int) get(this.impl().m_id) set(const_cast< unsigned int & >(this.impl().m_id))
    #pragma db member(name) virtual(std::string) access(this.impl().m_name)
    #pragma db member(employees) virtual(Employees) access(this.impl().m_employees)

    friend class odb::access;

public:
    ~Employer();

protected:
    class Impl;
    Employer();
    Employer(Impl *);

    const Impl & impl() const;
    Impl & impl();

private:
    Impl * const impl_;
};

#endif //TEST_H
